#include<stdio.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "config.h"
#include "vacek.h"
#include "prihradka.h"
#include "hra.h"

#define		TRUE		1
#define		FALSE		0

/* Vlastnosti hracch kamen */
#define ABOVE                   1
#define RIGHT                   2
#define BELOW                   4
#define LEFT                    8

/* --------------------- DEFINICE GLOBALNCH PROMNNCH --------------------- */

GLOBAL skamen board[MAX_ROZMER][MAX_ROZMER];  /* deska */

/* ---------------------- DEFINICE LOCALNCH PROMNNCH --------------------- */

LOCAL boolean board_click;                    /* Oznaeno pole na desce? */

/* --------------- PLN FUNKN PROTOTYPY GLOBALNCH FUNKC ---------------- */

GLOBAL boolean board_is_field_vacant(unsigned int x, unsigned int y);
GLOBAL boolean board_is_it_tile_from_rack(int x, int y);
GLOBAL boolean board_is_tile_joker(int x, int y);
GLOBAL boolean board_is_tile_clicked(void);
GLOBAL boolean board_is_tile_adjacent(int x, int y);
GLOBAL void board_set_field_vacant(int x, int y, boolean status);
GLOBAL void board_set_tile_clicked(boolean status);	
GLOBAL void board_set_letter_at_coords(int x, int y, char znak);

/* ---------------- PLN FUNKN PROTOTYPY LOCALNCH FUNKC ---------------- */

/* Vrac TRUE, je-li pole na desce voln */
GLOBAL boolean board_is_field_vacant(unsigned int x, unsigned int y) {
    
    return((board[x+1][y+1].znak == EMPTY_TILE) ? TRUE : FALSE);
}

/* -------------------------------------------------------------------------- */

/* Nastav 'board_click' na TRUE/FALSE 
 * (meme/nememe pole na desce pesouvat)
*/
GLOBAL void board_set_tile_clicked(boolean status) {

    board_click = status;
}

/* -------------------------------------------------------------------------- */

/* Vrac TRUE, je-li kmen na [x,y] olkem */
GLOBAL boolean board_is_tile_joker(int x, int y) {

    skamen *rack;
    int what;

    /* Vyber ten sprvn rack */
    rack = (player == HRAC1) ? rack1 : rack2;

    what = rack_which_tile_at_coords(x, y);

    if (rack[what].znak == JOKERCHAR)
	return(TRUE);
    else
	return(FALSE);

}/* end of board_is_tile_joker() */

/* -------------------------------------------------------------------------- */

/* Vrac TRUE, nachz-li se na [x,y] psmeno prv umstn z racku */
GLOBAL boolean board_is_it_tile_from_rack(int x, int y) {

    int i;

    for (i=0; i<MAX_PISMEN_PRIHRADKA; i++) {
	if ((rack_placed_tiles_coords[i].x == x) && 
	    (rack_placed_tiles_coords[i].y == y))
	    return (TRUE);
    }
	
    return(FALSE);
}

/* -------------------------------------------------------------------------- */

/* Vrac znak na pozici x, y */
GLOBAL char board_get_letter_at_coords(int x, int y) {

    return(board[x+1][y+1].znak);

}/* end of board_get_letter_at_coords() */

/* -------------------------------------------------------------------------- */

/* Umst na danou pozici na desce psmeno. Funkce je pouita pi
 * prci se olky.
*/
GLOBAL void board_set_letter_at_coords(int x, int y, char znak) {

    board[x+1][y+1].znak = znak;

}/* end of board_set_letter_at_coords() */

/* -------------------------------------------------------------------------- */

/* Nastav pole na desce na TRUE/FALSE (przdn/obsazen) a umst na
 * dan souadnice v board[][] pslun psmeno */
GLOBAL void board_set_field_vacant(int x, int y, boolean status) {

    int i;
    skamen *rack;

    if (status == TRUE) {		/* Przdn = -1 */
	board[x+1][y+1].znak = EMPTY_TILE;
	board[x+1][y+1].hodnota = EMPTY_TILE;
    }
    else {                              /* Zaplnme pole na boardu */
	i = rack_which_tile_at_coords(x, y);
	rack = (player == HRAC1) ? rack1 : rack2; /* Nastav sprvn rack */
	board[x+1][y+1].znak = rack[i].znak;
	board[x+1][y+1].hodnota = rack[i].hodnota;
    }
    
}

/* -------------------------------------------------------------------------- */

/* Vrac TRUE, je-li kliknuto na pole na desce, kter je z racku (pesun) */
GLOBAL boolean board_is_tile_clicked(void) {

    return(board_click);    
}

/* -------------------------------------------------------------------------- */

/* Zkontroluje, zda je prv odehrvan kmen na [x,y] sousedc
 * s ji odehranm */
GLOBAL boolean board_is_tile_adjacent(int x, int y) {
   
    int direction = ABOVE | RIGHT | BELOW | LEFT; 
    /* budeme testovat vechny smry */

    /* x-ova je 0 => lev okraj, 'vlevo' nic urit nen */
    direction = ( x == 0 ) ? direction^LEFT : direction;
    /* y-ova je 0 => horn okraj, 'nad' nic urit nen */
    direction = ( y == 0 ) ? direction^ABOVE : direction;
    /* x-ova je 15 => prav okraj, 'vpravo' nic urit nen */
    direction = ( x == POC_KAMENU_SIRKA-1) ? direction^RIGHT : direction;
    /* y-ova je 15 => doln okraj, 'pod' nic urit nen */
    direction = ( y == POC_KAMENU_VYSKA-1) ? direction^BELOW : direction;
  
    /* Pozor, nutno tak otestovat, zda nen sousedc kmen jeden
     * z prv umsovanch kamen!!!
    */
    if ((direction & ABOVE) && 
	(board_is_field_vacant(x, y-1) == FALSE) &&	
	(rack_is_it_currently_played_tile(x, y-1) == FALSE))
	return(TRUE);
    
    if ((direction & RIGHT) && 
	(board_is_field_vacant(x+1, y) == FALSE) &&
	(rack_is_it_currently_played_tile(x+1, y) == FALSE))
	return(TRUE);
    
    if ((direction & BELOW) && 
	(board_is_field_vacant(x, y+1) == FALSE) &&
	(rack_is_it_currently_played_tile(x, y+1) == FALSE))
	return(TRUE);
    
    if ((direction & LEFT) && 
	(board_is_field_vacant(x-1, y) == FALSE) &&
	(rack_is_it_currently_played_tile(x-1, y) == FALSE))
	return(TRUE);
    
    printf("kamen [%d,%d] nen pilehl!\n", x, y);
    return(FALSE);
    
}

/* -------------------------------------------------------------------------- */

/* END_OF_FILE deska.c */
