#include<stdlib.h>
#include<gnome.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "config.h"
#include "cross.h"
#include "vacek.h"
#include "prihradka.h"
#include "deska.h"
#include "skore.h"
#include "tahy_gui.h"
#include "gui_okno.h"
#include "gui_deska.h"
#include "hra.h"
#include "util.h"
#include "cesty.h"

/* Formt ukldanho souboru:
 * --------------------------
 *
 * Nzev poloky	Typ		Popis hodnoty
 * -------------	---		-------------------------------------
 * SIRKA_DESKY		int		poet kamen na ku
 * VYSKA_DESKY		int		poet kamen na vku
 * POC_PRIHRADKA	int		max. poet kamen v pihrdce
 * SLOVNIK_EXT		uchar		(standardn/extern slovnk? (0/1)
 * SLOVNIK_DELKA	uchar		dlka cesty k ext. slovnku 
 * 					(jinak 0/1 - Scrabble/Kovksk)
 * SLOVNIK		char *		cesta k externmu slovnku (.mat)
 * 					(jinak poloka chyb)
 * PL1_JMENO_DELKA	uchar		dlka jmna hre 1
 * PL1_JMENO		char *		jmno hre 1
 * PL2_JMENO_DELKA	uchar		dlka jmna hre 2
 * PL2_JMENO		char *		jmno hre 2
 * PL1_SKORE		int		aktuln skre hre 1
 * PL2_SKORE		int		aktuln skre hre 2
 * SADA_PISMEN_POC	int		poet psmen v sad
 * SADA_PISMEN		{char,int}	(znak, bodov ohodnocen)
 * SADA_INDEXU		{int,uchar}	(index do pole, pznak mnitelnosti)
 * SADA_I_INDEX		int		index v poli nhodnch hodnot (ukvacek)
 * POCITAC		uchar		kter z hr je 'Pota'? (0/1)
 * AKT_HRAC		uchar		kter z hr je aktuln?
 * PASS_POCET		uchar		kolik nsledovnch PASS u bylo? 
 * DESKA		{char,int}	(znak, hodnota) (SIRKA x VYSKA krt)
 * CROSS		{vcross}	(SIRKA x VYSKA - cross-set mnoiny)
 * SCORE		{dvojice}	(SIRKA x VYSKA - score pro /s)
 * PMULT		{char}		(SIRKA x VYSKA - psmenn bonusy)
 * SMUL			{char}		(SIRKA x VYSKA - slovn bonusy)
 * -------------------------------------------------------------------------
 *
 * Nsleduje seznam provedench tah:
 * ----------------------------------
 * 
 * TAHY_DELKA		int		poet provedench tah (z prubeh_clistu)
 * TAHY			structTAH
 *
 * structTAH = 	{ char tah_TYP, 
 * 			    ('h'-hrno, 'v'-vmna, 'p'-pass, 'o'-odpis)
 * 		    tahINFO:	   
 * 		            (bli info o tahu)
 * 		    case h: {uchar, uchar, uchar, int, uchar, char *} 
 * 		    	    (orientace, X, Y, body, dlka slova, slovo)
 * 		    case v: {uchar} 
 * 		    	    (poet vymnnch kamen)
 * 		    case p: EMPTY
 * 		    case o: {uchar, int} 
 * 		    	    (slo hre, suma odpisu)
 * 		}
 *
*/
/* --------------------- DEFINICE GLOBALNCH PROMNNCH --------------------- */

/* --------------------- DEFINICE LOCALNCH PROMNNCH ---------------------- */

LOCAL gchar *nazev_souboru;
LOCAL GtkWidget *file_box;
LOCAL FILE *SOUBOR;

/* ---------------- PLN FUNKN PROTOTYPY GLOBALNCH FUNKC --------------- */

GLOBAL void gui_save_game_callback(void);

/* ----------------- PLN FUNKN PROTOTYPY LOCALNCH FUNKC --------------- */

LOCAL void game_save_proved_ulozeni(GtkFileSelection *vyber, gpointer data);
LOCAL void game_save_zapis_retezec(char *retezec);
LOCAL void game_save_zapis_odehrany(int poradi);
LOCAL void game_save_zapis_vymenu(int poradi);
LOCAL void game_save_zapis_pass(void);
LOCAL void game_save_zapis_odpis(int poradi);

/* ---------------------- DEFINICE GLOBALNCH FUNKC ------------------------ */

/* Oteve dialog pro uloen souboru se stavem hry */
GLOBAL void gui_save_game_callback(void) {


    /* Neukldej, nen-li co */
    if (game_mode != PLAY_MODE)
	return;

    file_box = gtk_file_selection_new("Uloit stav hry.");
    gtk_file_selection_complete(GTK_FILE_SELECTION(file_box), ".sca");
    gtk_signal_connect(
	    GTK_OBJECT(GTK_FILE_SELECTION(file_box)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC(game_save_proved_ulozeni),
		      (gpointer) file_box);
	       
    gtk_signal_connect_object(
	    GTK_OBJECT(GTK_FILE_SELECTION(file_box)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
		      (gpointer) file_box);

    gtk_signal_connect_object(
	    GTK_OBJECT(GTK_FILE_SELECTION(file_box)->cancel_button),
		      "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
		      (gpointer) file_box);

    gtk_widget_show (file_box);
    gtk_window_set_modal(GTK_WINDOW(file_box), TRUE);

}/* end gui_save_game() */

/* ----------------------- DEFINICE LOCALNCH FUNKC ------------------------ */

/* Zskej nzev souboru z file_selectoru */
LOCAL void game_save_proved_ulozeni(GtkFileSelection *vyber, gpointer data) {

    int i, j, k;
    char znak;
    unsigned char byte;
    unsigned int lo, hi;
    int skore;


    /* Nutnost! Nebo je-li kmen z pihrdky na desce, je i v board[][]!!! */
    rack_return_played_tiles_home();
    
    nazev_souboru = 
	gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_box));

    printf("Nzev souboru = '%s'\n", nazev_souboru);    
 
    /* Ulo pod sprvnm nzvem */
    if ( (util_soubor_ma_priponu(nazev_souboru, ".sca")) == FALSE )
	strcat(nazev_souboru, ".sca");

    /* Vybafni okno, pokud dojde k chyb */
    if ( (SOUBOR = fopen(nazev_souboru, "wb")) == NULL ) {
	fprintf(stderr, "Soubor '%s' se nepodailo otevt pro zpis!\n",
		nazev_souboru);
	show_warning_window(GTK_WINDOW(app), 
			"Soubor se nepodailo otevt pro zpis!");
	return;
    }

    /* Njak optimalizace? */    
    if (setvbuf(SOUBOR, NULL, _IOFBF, 8192) != 0)
	fprintf(stderr, 
		"(gui_save.c) Chyba pi nastaven blokovho bufferovn!");


   
    /* ------ NAPLNME SOUBOR ------- */ 
    
    /* A zaneme zapisovat do souboru */

    /* Deska */
    fwrite(&POC_KAMENU_SIRKA, sizeof(POC_KAMENU_SIRKA), 1, SOUBOR);
    fwrite(&POC_KAMENU_VYSKA, sizeof(POC_KAMENU_VYSKA), 1, SOUBOR);
    fwrite(&MAX_PISMEN_PRIHRADKA, sizeof(MAX_PISMEN_PRIHRADKA), 1, SOUBOR);

    /* Extern/standardn slovnk? (1/0) */
    byte = ext_slovnik;
    fwrite(&byte, sizeof(byte), 1, SOUBOR);
    
    /* Cesta k externmu/standardnmu slovnku */
    if (ext_slovnik == TRUE)
	game_save_zapis_retezec(akt_slovnik_cesta);
    else
	game_save_zapis_retezec(STD_SLOVNIK_CESTA);

    /* Jmna hr */
    game_save_zapis_retezec(gtk_entry_get_text(GTK_ENTRY(player1_name)));
    game_save_zapis_retezec(gtk_entry_get_text(GTK_ENTRY(player2_name)));

    /* Jejich skore */
    fwrite(&skore_h1, sizeof(skore_h1), 1, SOUBOR);
    fwrite(&skore_h2, sizeof(skore_h2), 1, SOUBOR);

    /* Sada kamen */

    /* Jejich poet */
    i = vacek_pocet_kamenu();
    fwrite(&i, sizeof(i), 1, SOUBOR);

    /* Soubor kamen [znak, body] */
    for(j=0;j<i;j++) {
	znak = sada_kamenu[j].znak;
	k = sada_kamenu[j].hodnota;
	fwrite(&znak, sizeof(znak), 1, SOUBOR);
	fwrite(&k, sizeof(k), 1, SOUBOR);
    }/* end for (j) */

    /* Nhodn indexy. Je jich stejn jako vech kamen. */
    for(j=0;j<i;j++) {
	k = vacek[j].poradi;
	byte = vacek[j].menitelny;
	fwrite(&k, sizeof(k), 1, SOUBOR);
	fwrite(&byte, sizeof(byte), 1, SOUBOR);
    }/* end for (j) */

    /* Pozice ve vku - ukvacek (vacek.c) */
    i = vacek_dej_pozici_ve_vacku();
    fwrite(&i, sizeof(i), 1, SOUBOR);
   
    /* Kter z hr je pota (0/1/2 - 1./2./dn */
    byte = computer;
    fwrite(&byte, sizeof(byte), 1, SOUBOR);

    /* Aktuln hr */
    byte = player;
    fwrite(&byte, sizeof(byte), 1, SOUBOR);

    /* Kdo zanal? */
    byte = hra_dej_zacinal_hrac();
    fwrite(&byte, sizeof(byte), 1, SOUBOR);

    /* Poet po sob nsledujcch PASS ve he */
    byte = pass_count;
    fwrite(&byte, sizeof(byte), 1, SOUBOR);
    

    /* Uloen cel desky */
    for(i=1;i<=POC_KAMENU_SIRKA;i++) {
	for(j=1;j<=POC_KAMENU_VYSKA;j++) {
	    znak = board[i][j].znak;
	    k = board[i][j].hodnota; 
	    fwrite(&znak, sizeof(znak), 1, SOUBOR);
	    fwrite(&k, sizeof(k), 1, SOUBOR);
	    printf("%c", (znak != EMPTY_TILE) ? znak : '0');
	}/* end for (j) */
	printf("\n");
    }

    /* Ukldn cross-set mnoin (Sirka x Vyska x 2 (orientace) x 8 (2 inty)) */
    /* Pro klasick Scrabble = 15 * 15 * 2 * 8 = 3600 Bytes */
    for(i=1;i<=POC_KAMENU_SIRKA;i++)
	for(j=1;j<=POC_KAMENU_VYSKA;j++) {
	    lo = cross[i][j][ZD].lo;
	    hi = cross[i][j][ZD].hi;
	    fwrite(&lo, sizeof(lo), 1, SOUBOR);
	    fwrite(&hi, sizeof(hi), 1, SOUBOR);
	    lo = cross[i][j][SD].lo;
	    hi = cross[i][j][SD].hi;
	    fwrite(&lo, sizeof(lo), 1, SOUBOR);
	    fwrite(&hi, sizeof(hi), 1, SOUBOR);
	}/* end for (j) */

    /* Ukldn struktur pro vpoet skore */
    /* Pro klasick Scrabble = 15 * 15 * 2 * 4 = 1800 Bytes */
    for(i=1;i<=POC_KAMENU_SIRKA;i++)
	for(j=1;j<=POC_KAMENU_VYSKA;j++) {
	    skore = score[i][j][ZD];
	    fwrite(&skore, sizeof(skore), 1, SOUBOR);
	    skore = score[i][j][SD];
	    fwrite(&skore, sizeof(skore), 1, SOUBOR);
	}/* end for (j) */

    /* Ukldn psmennch bonus - pole char */
    /* Pro klasick Scrabble = 15 * 15 * 1 = 225 Bytes */
    for(i=1;i<=POC_KAMENU_SIRKA;i++)
	for(j=1;j<=POC_KAMENU_VYSKA;j++) {
	    znak = pismmult[i][j];
	    fwrite(&znak, sizeof(znak), 1, SOUBOR);
	}

    /* Ukldn slovnch bonus - pole char */
    /* Pro klasick Scrabble = 15 * 15 * 1 = 225 Bytes */
    for(i=1;i<=POC_KAMENU_SIRKA;i++)
	for(j=1;j<=POC_KAMENU_VYSKA;j++) {
	    znak = slovmult[i][j];
	    fwrite(&znak, sizeof(znak), 1, SOUBOR);
	}

    /* Ulome tahy */
    i = prubeh_dej_pocet();
    fwrite(&i, sizeof(i), 1, SOUBOR);	/* Dlka */
    j = 0; /* 'b' je kdo zaal */
    while(j<i) {

	/* Typ tahu */
	znak = prubeh_dej_typ(j);

	switch(znak) {
	    case 'h':	game_save_zapis_odehrany(j); break;
	    case 'v':	game_save_zapis_vymenu(j);   break;
	    case 'p':	game_save_zapis_pass();	     break;
	    case 'o':	game_save_zapis_odpis(j);    break;
	}/* end switch */

	j++;

    }/* end while */

    /* Zapi buffery na disk */
    fclose(SOUBOR);
    
}/* end game_save_proved_ulozeni() */

/* -------------------------------------------------------------------------- */

/* Zape do souboru nejprve 'unsigned char' jako dlku etzec a
 * nsledn zape sadu 'char' z etzce.
*/
LOCAL void game_save_zapis_retezec(char *retezec) {

    unsigned char i;
    
    i = strlen(retezec);
    fwrite(&i, sizeof(i), 1, SOUBOR);	       /* Dlka */
    fwrite(retezec, i*sizeof(char), 1, SOUBOR);/* Samotn text */

}/* end of game_save_zapis_retezec() */

/* -------------------------------------------------------------------------- */

/* Zape do souboru 'h' (Hrno), dle
 *  uchar orientace (0/1)
 *  uchar, uchar souadnice X,Y
 *  int body
 *  uchar slovo_delka
 *  char * slovo
*/
LOCAL void game_save_zapis_odehrany(int poradi) {
    
    unsigned char byte;
    unsigned char o = HORIZONTAL;
    int j, k, l;
    char *s = NULL;
    char t[5];
    char znak = 'h';

    /* Typ tahu */
    fwrite(&znak, sizeof(znak), 1, SOUBOR);

    /* Souadnice */
    gtk_clist_get_text(GTK_CLIST(prubeh_clist), poradi, 3, &s);
    
    j = k = 0;

    /* zanaj psmenem? -=> vertikln tah */ 
    if (s[0] > '9')
	o = VERTICAL;
   
    /* Zapi orientaci */
    fwrite(&o, sizeof(o), 1, SOUBOR); 
    
    if (o == VERTICAL) {	/* Mme nap. H12 */
	j = k = 0;
	byte = s[j++] - 'A' + 1; 
	
	fwrite(&byte, sizeof(byte), 1, SOUBOR);

	while(s[j] <= '9' && s[j] >= '0')	/* Nati veker slice */
	    t[k++] = s[j++];
	t[k] = '\0';
	l = atoi(t);
	byte = l;

	fwrite(&byte, sizeof(byte), 1, SOUBOR);	/* 15 */

    }
    else {	/* Mme nco jako '15H' */
	j = k = 0;
	while(s[j] <= '9' && s[j] >= '0')	/* Nati veker slice */
	    t[k++] = s[j++];
	t[k] = '\0';
	l = atoi(t);
	byte = l;

	fwrite(&byte, sizeof(byte), 1, SOUBOR);	/* 15 */

	byte = s[j] - 'A' + 1; 

	fwrite(&byte, sizeof(byte), 1, SOUBOR);	/* H = ABCDEFGH = 12345678 = 8*/
    }

    /* Body */
    gtk_clist_get_text(GTK_CLIST(prubeh_clist), poradi, 2, &s);
    l = atoi(s);
    fwrite(&l, sizeof(l), 1, SOUBOR);
    
    /* Slovo */
    gtk_clist_get_text(GTK_CLIST(prubeh_clist), poradi, 4, &s);

    game_save_zapis_retezec(s);
    
}/* end of game_save_zapis_odehrany() */

/* -------------------------------------------------------------------------- */

/* Zape do souboru znak 'v' a 'uchar' jako poet vymnnch kamen */
/* Poet vymnnch znak se zsk prascky z clistu a je to etzec mezi
 * zvorkama '(XXXX)'.
*/
LOCAL void game_save_zapis_vymenu(int poradi) {

    char *s = NULL;
    char znak = 'v';
    unsigned char i;
    int j = 0;
    int k = 0;
    char pocet[5] = "";

    fwrite(&znak, sizeof(znak), 1, SOUBOR);
    gtk_clist_get_text(GTK_CLIST(prubeh_clist), poradi, 4, &s);

    /* Najdi lev zvorky */
    while(s[j++] != '(');

    /* A vysosej ve a do pravch */
    while(s[j] != ')')
	pocet[k++] = s[j++];
    
    pocet[k] = '\0';

    printf("Poradi %d = vymena '%s'\n", poradi, pocet);

    k = atoi(pocet);
    i = k;

    /* Zpis samotnho potu vymnnch kamen */
    fwrite(&i, sizeof(i), 1, SOUBOR);
    
}/* end of game_save_zapis_vymenu() */

/* -------------------------------------------------------------------------- */

/* Zape do souboru znak 'p' jako 'Pass' */
LOCAL void game_save_zapis_pass(void) {

    char znak = 'p';

    fwrite(&znak, sizeof(znak), 1, SOUBOR);

}/* end of game_save_zapis_pass() */

/* -------------------------------------------------------------------------- */

/* Zapi do souboru znak 'o' a odpisovanou hodnotu int */
LOCAL void game_save_zapis_odpis(int poradi) {

    char znak;
    int i;

    znak = 'o';
    i = 111;

    fwrite(&znak, sizeof(znak), 1, SOUBOR);
    fwrite(&i, sizeof(i), 1, SOUBOR);

}/* end of game_save_zapis_odpis() */

/* -------------------------------------------------------------------------- */

/* END_OF_FILE gui_save.c */
