#include<stdio.h>
#include "headers/stddfn.h"
#include "headers/globals.h"
#include "config.h"
#include "automat.h"
#include "cross.h"
#include "skore.h"
#include "vacek.h"
#include "prihradka.h"
#include "deska.h"
#include "konverze.h"
#include "tahy_seznam.h"
#include "zakazana_seznam.h"
#include "hra.h"



#define TRUE	1
#define FALSE	0

/* ----------------- DEKLARACE GLOBALNCH PROMNNCH ------------------------ */
/* ------------------ DEKLARACE LOCALNCH PROMNNCH ------------------------ */

/* Pro poteby hledn tahu, dky nad a pod, aktuln dek a cross */
LOCAL skamen rad_akt[MAX_ROZMER];
LOCAL skamen rad_nad[MAX_ROZMER];
LOCAL skamen rad_pod[MAX_ROZMER];
LOCAL tcross rad_cross[MAX_ROZMER];

LOCAL int rad_score[MAX_ROZMER];
LOCAL int rad_smult[MAX_ROZMER];
LOCAL int rad_pmult[MAX_ROZMER];
LOCAL int tahy_rack[MAX_VEL_ABECEDY];


LOCAL int kameny;		 /* Poet pouitch kamen pi hledn tahu */
LOCAL int pouzite_kameny[MAX_ROZMER]; /* A kter to jsou (indexy) */
LOCAL int tah_skoreCelkem;		  /* Do nj skre za tah */

/* --------------------- DEKLARACE GLOBALNCH FUNKC ------------------------ */

GLOBAL void tahy_generuj_tah(void);
GLOBAL void tahy_generuj_prvni_tah(void);
GLOBAL int tahy_spocitej_body(int hrac, tpozice p, torientace o);
GLOBAL int POCETTAHU = 0;
/* ---------------------- DEKLARACE LOCALNCH FUNKC ------------------------ */

LOCAL void tahy_gen_napln_struktury(torientace o, int radek);
LOCAL void tahy_napln_tahy_rack(void);
#ifdef TAHY_DEBUG
LOCAL void tahy_cross_vypis(int sloupec);
LOCAL void vypisujky(torientace o, int radek);
#endif
LOCAL void findmove(torientace o, int radek);
LOCAL void prefix(Stav prechod, int maxdelka, int sloupec, int radek, 
	          torientace o);
LOCAL void extend(Stav prechod, int sloupec, int radek, torientace o);
LOCAL boolean tahy_je_v_cross(int sloupec, char znak);
LOCAL void eval(int sloupec, int radek, torientace o, boolean pridej);
/* -------------------------------------------------------------------------- */

#ifdef TAHY_DEBUG
LOCAL void vypisujky(torientace o, int radek) {

    int i, maximum;

    if (o == ZD)
	maximum = POC_KAMENU_SIRKA;
    else
	maximum = POC_KAMENU_VYSKA;

    printf("--- ORIENTACE = %s ---\n", (o==ZD) ? "HORIZONTAL" : "VERTICAL");
    printf("--- RADEK = %d\n", radek);
    
    printf("--- ABOVE = ");
    for(i=1;i<=maximum;i++)
	printf("%c", (rad_nad[i].znak != EMPTY_TILE) ? rad_nad[i].znak : '0');
    printf("\n");
    printf("--- FILL  = ");
    for(i=1;i<=maximum;i++)
	printf("%c", (rad_akt[i].znak != EMPTY_TILE) ? rad_akt[i].znak : '0');
    printf("\n");
    printf("--- BELOW = ");
    for(i=1;i<=maximum;i++)
	printf("%c", (rad_pod[i].znak != EMPTY_TILE) ? rad_pod[i].znak : '0');
    printf("\n");
    printf("--- PISMMULT = ");
    for(i=1;i<=maximum;i++)
	printf("%d", rad_pmult[i]);
    printf("\n");
    printf("--- SLOVMULT = ");
    for(i=1;i<=maximum;i++)
	printf("%d", rad_smult[i]);
    printf("\n");
    
}/* end vypisujky() */
#endif
/* -------------------------------------------------------------------------- */
#ifdef TAHY_DEBUG
LOCAL void tahy_cross_vypis(int sloupec) {

    char s[256] = "";

    int i, k=0;
    unsigned long int maska=0;

    for(i=0;i<S_BYTE*S_ULONG;i++) {
        maska = 1 << i;
        if (maska & rad_cross[sloupec].lo) {
            s[k] = ABECEDA[i];
            k++;
        }
    }
    for(i=0;i<S_BYTE*S_ULONG;i++) {
        maska = 1 << i;
        if (maska & rad_cross[sloupec].hi) {
            s[k] = ABECEDA[S_BYTE*S_ULONG+i];
            k++;
        }
    }

    /* Ukonme korektn etzec */
    s[k] = '\0';

    //printf("Pro [%d] = '%s'\n", sloupec, s);



}/* end of tahy_cross_vypis() */
#endif
/* -------------------------------------------------------------------------- */


/* Napln postupn rad_* pro dky nad, aktuln a pod a cross-set
 * mnoinu pro danou adu na desce a orientaci.
 * Dle pevedeme pihrdku pro pouit v algoritmu, v poli ABECEDA mme
 * vechny povolen znaky, vytvome 'pole int' o velikosti abecedy
 * (+1 - JOKER) a hodnota 'pole[i]' bude oznaovat poet psmen na indexu 'i' 
 * v poli ABECEDA, kter momentln mme v pihrdce.
 *
 * Pklad: Pihrdka obsahuje 'AABBCC'.
 *          pole[0] (0 == index 'A' v ABECEDA) bude mt hodnotu 2,
 *          pole[4] == 2
 *          pole[5] == 2
 *
 *          Nebo ABECEDA = abc...
 *                          012345
*/	          

LOCAL void tahy_gen_napln_struktury(torientace o, int radek) {

    int i, maximum;
    tpozice p;

    /* Vynuluj pomocn pole s psmeny, kter umstme na desku */
    for(i=0;i<=MAX_PISMEN_PRIHRADKA;i++)
	pouzite_kameny[i] = EMPTY_TILE;

    /* Nepouity dn kameny z pihrdky pro nalezen tahu */
    kameny = 0;

    p.x = (o == ZD) ? radek : 1;
    p.y = (o == ZD) ? 1 : radek;

    /* Nejdve vechno vynulujeme */
    for(i=0;i<MAX_ROZMER;i++) {
	rad_nad[i].znak = EMPTY_TILE; rad_nad[i].hodnota = EMPTY_TILE;
	rad_akt[i].znak = EMPTY_TILE; rad_akt[i].hodnota = EMPTY_TILE;
	rad_pod[i].znak = EMPTY_TILE; rad_pod[i].hodnota = EMPTY_TILE;
	rad_cross[i].lo = 0; rad_cross[i].hi = 0;
	rad_score[i] = 0;
    }

    /* Bonusov pole */
    for(i=0;i<MAX_ROZMER;i++) {
	rad_pmult[i] = 0;
	rad_smult[i] = 0;
    }

    /* A naplnme sprvnmi hodnotami */
    if (o == ZD)
	maximum = POC_KAMENU_SIRKA;
    else
	maximum = POC_KAMENU_VYSKA;

    for(i=1;i<=maximum;i++) {
	rad_nad[i].znak	   = PISMENO(PRED(p, KOLMO(o)));		
	rad_nad[i].hodnota = HODNOTA(PRED(p, KOLMO(o)));
	rad_akt[i].znak    = PISMENO(p);
	rad_akt[i].hodnota = HODNOTA(p);
	rad_pod[i].znak    = PISMENO(ZA(p, KOLMO(o)));		
	rad_pod[i].hodnota = HODNOTA(ZA(p, KOLMO(o)));
	rad_cross[i] = cross[p.x][p.y][KOLMO(o)];
	rad_score[i] = score[p.x][p.y][KOLMO(o)];
	rad_pmult[i] = pismmult[p.x][p.y];
	rad_smult[i] = slovmult[p.x][p.y];
	/* Posu souadnice ve sprvnm smru */
	p=ZA(p,o);
    }

}/* end tahy_gen_napln_struktury() */

/* -------------------------------------------------------------------------- */

/* Pevede psmennou pihrdku na etezec index */
LOCAL void tahy_napln_tahy_rack(void) {

    int i, j;
    skamen *rack;

    /* Naplnme rack 'nulami' */
    for(i=0;i<MAX_VEL_ABECEDY;i++)
	tahy_rack[i] = 0;

    /* Vyber ten sprvn rack */
    rack = (player == HRAC1) ? rack1 : rack2;
   
    /* Zvy potadlo vskyt pro kad psmeno z pihrdky */
    for(i=0;i<MAX_PISMEN_PRIHRADKA;i++) {
	if ((j=rack[i].znak) != EMPTY_TILE) 
	    tahy_rack[znak2index(j)]++;
    }/* end for */

}/* end of tahy_napln_tahy_rack() */

/* -------------------------------------------------------------------------- */

/* Mm aktuln dek a cross-set mnoinu pro danou orientaci.
 * Zjisti, zda je mon umstit znak 'znak' na dan sloupec 'sloupec'.
*/
LOCAL boolean tahy_je_v_cross(int sloupec, char znak) {

    int i, blowhi=FALSE;
    unsigned long maska=0;

    for(i=0; i<MAX_VEL_ABECEDY-1; i++)
	if (ABECEDA[i] == znak)		/* Najdi index psmene v abeced */
	    break;

    if (i == MAX_VEL_ABECEDY-1)         /* Nenali jsme toto psmeno => FALSE */
        return(FALSE);


    if (i >= S_BYTE*S_ULONG) {
        i -= S_BYTE*S_ULONG;
        blowhi = TRUE;
    }

    maska = 1 << i;

    if (blowhi)
        return ( (rad_cross[sloupec].hi & maska) ? TRUE : FALSE);
    else
        return ( (rad_cross[sloupec].lo & maska) ? TRUE : FALSE);

    return(FALSE);
    
}/* end tahy_je_v_cross() */

/* -------------------------------------------------------------------------- */

/* Hledn tah, projde kadou adu na desce (horizontln/vertikln) a
 * nalezne vechnt mon tahy. Nejdleitj voln celho programu :-)
*/
GLOBAL void tahy_generuj_tah(void) {

    int i;
    torientace o;

    /* Generujeme nov seznam tah */
    tahy_seznam_uvolni();

    POCETTAHU=0;

    /* Volme jen 1x - sta */
    tahy_napln_tahy_rack();
   
#ifdef GEN_DEBUG 
    printf("--- RACK ---\n");
    for(i=0;i<MAX_VEL_ABECEDY-1;i++)
	printf("%c",ABECEDA[i]);
    printf("\n");
    for(i=0;i<MAX_VEL_ABECEDY-1;i++)
	printf("%d", tahy_rack[i]);
    printf("\n");
#endif

    /* ZlevaDoprava - horizontln */
    o=ZD;
    for(i=1;i<=POC_KAMENU_SIRKA;i++)
	findmove(o, i);
    
    /* ShoraDol - vertikln */
    o=SD;
    for(i=1;i<=POC_KAMENU_VYSKA;i++)
	findmove(o, i);

#ifdef GEN_DEBUG
    
    printf("TAH KONEC\n");    

    tahy_seznam_vypis_seznam();

    printf("TAH KONEC2\n");

#endif

}/* end tahy_generuj_tah() */



/* -------------------------------------------------------------------------- */

/* Generuj tah pes stedov pole, pes kter vdy mus prvn tah prochzet */
GLOBAL void tahy_generuj_prvni_tah(void) {
    
    int i, MIDDLE_R, MIDDLE_S;
    torientace o=ZD;

    /* Generujeme nov seznam tah */
    tahy_seznam_uvolni();

    POCETTAHU=0;

    /* Volme jen 1x - sta */
    tahy_napln_tahy_rack();
   
#ifdef GEN_DEBUG
    
    printf("--- RACK ---\n");
    for(i=0;i<MAX_VEL_ABECEDY-1;i++)
	printf("%c",ABECEDA[i]);
    printf("\n");
    for(i=0;i<MAX_VEL_ABECEDY-1;i++)
	printf("%d", tahy_rack[i]);
    printf("\n");
#endif

    /* Spoteme prostedn dek na desce */
    i = POC_KAMENU_VYSKA;      /* Z vky */
    MIDDLE_R = (i % 2) ? (i/2)+1 : (i/2);
    
    /* Napl rad_* dky z desky */
    tahy_gen_napln_struktury(o, MIDDLE_R);
    
    /* Spoteme stedov sloupec v tomto dku */
    i = POC_KAMENU_SIRKA;
    MIDDLE_S = (i % 2) ? (i/2)+1 : (i/2);
    
    /* Nalezni tah prochzejc stedovm polem */
    prefix(koren, MIDDLE_S-2, MIDDLE_S, MIDDLE_R, o);

#ifdef GEN_DEBUG
    tahy_seznam_vypis_seznam();
#endif

}/* end of tahy_generuj_prvni_tah() */

/* -------------------------------------------------------------------------- */



/* -------------------------------------------------------------------------- */
/* ------------- SEKCE FINDMOVE(), PREFIX(), EXTEND() a EVAL() -------------- */
/* -------------------------------------------------------------------------- */

/*
 * Najde tah pro danou adu na desce v dan orientaci. Pouv k tomu zchytn
 * msta, tzv. "anchor squares", co jsou przdn pole sousedc s ji 
 * odehranm kamenem. Pouit algoritmus pochz od pn A. W. Appela a
 * G. J. Jacobsona.
*/
LOCAL void findmove(torientace o, int radek) {

    int anchor = 0;
    int c;
    int maximum;

    if (o == ZD)
	maximum = POC_KAMENU_SIRKA;
    else
	maximum = POC_KAMENU_VYSKA;

    /* Napl rad_* dky z desky */
    tahy_gen_napln_struktury(o, radek);
    
    /* Pes vechna pole v dku na desce*/
    for(c=1; c<=maximum; c++) {

	/* Je pole przdn a soused s neprzdnm? (== anchor) */
	if( (  rad_akt[c].znak == EMPTY_TILE) && 
	     ((rad_nad[c].znak != EMPTY_TILE) || 
	      (rad_pod[c].znak != EMPTY_TILE) || 
	    (rad_akt[c-1].znak != EMPTY_TILE) || 
	    (rad_akt[c+1].znak != EMPTY_TILE))  )
	{
#ifdef GEN_DEBUG	    
	    printf("ANCHOR [%d,%d]\n", radek, c); 
	    vypisujky(o, radek);
#endif


	    /* Je pole vlevo od anchoru przdn? Ano = generuj prefix */ 
	    if(rad_akt[c-1].znak == EMPTY_TILE)
		/* c-anchor-1 = max. dlka prefixu */
		/* je to poet pol mezi 2 anchory, aktulnm a pedchozm */
		prefix(koren, c-anchor-1, c, radek, o);
	    else 
	    /* Pole vlevo nen przdn, me slovo zkusit rozit doprava */
		/* anchor+1 == anchor je posledn przdn pole, pokrauj tedy
		 * od pole od nj vpravo.
		*/
		extend(koren, anchor+1, radek, o);
	    
            /* aktualizuj posledn anchor */
	    anchor = c;
                        
        }/* end if () && () || .... */
	
    }/* end for */
    
}/* end findmove() */ 


/* -------------------------------------------------------------------------- */

/* Generuj vechny slovn pedpony dlky maximln 'maxdelka' z
 * pechodu 'prechod'. Pro kad nalezen takov prefix se jej
 * pokus rozit dle ve funkci 'extend'.
*/
LOCAL void 
prefix(Stav prechod, int maxdelka, int sloupec, int radek, torientace o) {

    int znak, d;
    Prechod n, m;
    Stav uzel;
    
    n.cis_hodnota = m.cis_hodnota = prechod;
    n.t.terminal = 0;	/* Nen koncov pechod, viz algoritmus */
    
    extend(n.cis_hodnota, sloupec, radek, o);

    /* 'n' m zmnn 'terminal' bit, pouijeme orig. kopii 'm' */
    uzel = m.t.do_stavu; 
  
    /* Prefix nulov dlky nehledej a nem-li kam pejt v grafu, skoni. */ 
    if (!maxdelka || !uzel)
	return;
    
    do {

	/* pouijeme opt 'n' */
	n.t = automat[uzel++].t;

	/* znak pro pechod */
	znak = znak2index(n.t.znak);

	/* doasn vyber psmeno z pihrdky */
	
	/* Mme 'znak' v pihrdce? */
	if (tahy_rack[znak])
	    tahy_rack[d=znak]--;	/* Ano, sni poet toho psmene */
	else if (tahy_rack[JOKER]) {	/* Mme JOKERa, pouij jej */
	    znak |= JOKERBIT;		/* Pouvme JOKERa */
	    tahy_rack[d=JOKER]--;	/* Pouili jsme JOKERa */
	}
	else 
	    continue;			/* Nemme JOKERa ani psmeno */
	
	    
	
		/* Ulo si, co jsme pouili za psmena */
		pouzite_kameny[kameny++] = znak;
		prefix(n.cis_hodnota, maxdelka-1, sloupec, radek, o);
		kameny--;	/* Pouit kameny */
		/* To, co jsme vythli z pihrdky, vra zpt */
		tahy_rack[d]++;
	    
    } while (!n.t.posledni);
    
}/* end prefix() */

/* -------------------------------------------------------------------------- */

/* Proli jsme graf a do pechodu 'prechod' a sname se pokraovat
 * dle v dku. 'sloupec' oznauje nsledujc pole, kam se budeme
 * snait rozit nalezen prefix.
*/
LOCAL void extend(Stav prechod, int sloupec, int radek, torientace o) {
    
    int znak, pznak, d;
    Prechod n;
    Stav uzel;

    n.cis_hodnota = prechod;
    uzel = n.t.do_stavu;

    /* zskej znak z aktulnho sloupce */ 
    znak = znak2index(rad_akt[sloupec].znak);

    /* Je mon, e jsme nali platn slovo */
    if ((n.t.terminal) && (znak==EMPTY_TILE))
	eval(sloupec, radek, o, TRUE);/* ohodno skre a tah */

    /* Meme pokraovat dl v grafu? */ 
    if (!uzel)
	return;

    if (znak == EMPTY_TILE) {	
    /* v danm sloupci nen dn psmeno, projdeme tedy vechny hrany
     * z naeho uzlu a zkontrolujeme, zda psmena na hranch je mon
     * na tento sloupec umstit (tj. existuj v "cross-set" mnoin).
    */
	do {

	    n.t = automat[uzel++].t;
	    pznak = znak2index(n.t.znak);
	   
	    if (tahy_je_v_cross(sloupec, n.t.znak)) {

		if (tahy_rack[pznak])	      /* Mme 'znak' v pihrdce? */
		    tahy_rack[d=pznak]--;     /* Ano, sni poet toho psmene */
		else if (tahy_rack[JOKER]) {  /* Mme JOKERa, pouij jej */
		    pznak |= JOKERBIT;        /* Pouvme JOKERa */
		    tahy_rack[d=JOKER]--;     /* Pouili jsme JOKERa */
		}
		else 
		    continue;	              /* Nemme JOKERa ani psmeno */
		
		    pouzite_kameny[kameny++] = pznak;
                    extend(n.cis_hodnota, sloupec+1, radek, o);
                    kameny--;
		    tahy_rack[d]++;
		    
	    }/* end if (cross) */
	    
       } while (!n.t.posledni);
	
    } else {
    /* Ve sloupci 'sloupec' je ji odehran psmeno. Nevybrej psmena
     * z pihrdky, ale pokus se pod odehranmi psmeny projt dle v 
     * grafu.
    */

	/* Projdi vechny pechody ze stavu a podvej se, zda nesouhlas
	 * psmeno z pechodu a znak na odehranm kameni.
	*/
	do {
	    n.t = automat[uzel++].t;
	    pznak = znak2index(n.t.znak);
	   
		/* Nali jsme hranu s psmenem z desky */
		if (pznak == znak) 
		    extend(n.cis_hodnota, sloupec+1, radek, o);
		
        } while (!n.t.posledni);
	
    }/* end else */
    
}/* end extend() */

/* -------------------------------------------------------------------------- */

/* Zjisti, co jsme nali a zaa/ohodno/vyhodno.
 * 'sloupec' bude przdn pole v dan ad, sousedc s prv nalezenm
 * slovem (bude za nm). Zrove se pln spojov seznam vech nalezench
 * tah. Parametr 'pridej' je tu kvli tomu, aby se neprovdlo pidn
 * tahu do seznamu tah pi pouhm potn skre za tah ve funkci
 * tahy_spocitej_body() - bugfix 24-03-2004.
*/
LOCAL void eval(int sloupec, int radek, torientace o, boolean pridej) {

#ifdef TAHY_GEN_DEBUG
    char testuj[MAX_ROZMER] = "";
    int j;
#endif

    char zakazane[MAX_ROZMER] = "";

    Tah t;		/* Budeme postupn plnit */

    int i, c, odehrany, pokladany;
    
    int skoreKolmehoSlova = 0;
    int skoreOdehranychPismenCelkem = 0;
    int skorePokladanehoPismene = 0;
    int nasobekPokladanehoSlova = 1;


    

    tah_skoreCelkem = 0;

    /* dlka vytvoenho slova + poet pouitch kamen z pihrdky.  */
    t.slen = t.klen = 0;

    /* Mus bt przdn */
    if (rad_akt[sloupec].znak != EMPTY_TILE) {
	fprintf(stderr, "FATAL eval(), chyba v algoritmu!!!\n");
	printf("FATAL sloupec = %d\n", sloupec);
	return;
    }

#ifdef TAH_GEN_DEBUG
    printf(" --- EVAL [%d,%d] ---\n", radek, sloupec);
#endif

    /* Nic jsme bohuel nepouili, nen tahem */    
    if (!kameny) {
	return;
    }

    c=sloupec;
   
    /* Projdeme zpt a podvme se, co jsme kam umstili */
    for (i=kameny-1; i>=0; i--) {
	
	/* Pesko na desce ji odehran kameny */
	while ((odehrany=rad_akt[--c].znak) != EMPTY_TILE) {
		t.s[t.slen++] = odehrany; 	/* psmeno ze slova */

		/* bugfix 07.02.2004 - pozor! Odehranm kamenem me
		 * bt olk - u nj nehledat bodov ohodnocen pes
		 * zastupujc psmeno, ale vzt pmo nulu!
		*/
		skoreOdehranychPismenCelkem += rad_akt[c].hodnota;
	/* odstranno 07.02.2004 - skore_vrat_hodnotu(znak2index(odehrany)); */
	}/* end while */

	/* Dt pry JOKERBIT, a vme, co zastupoval */
	pokladany = index2znak(pouzite_kameny[i] & ~JOKERBIT);	

	/* psmeno z pokldanho kamene */	
	t.k[t.klen].x = (o == ZD) ? radek : c;
	t.k[t.klen].y = (o == ZD) ? c : radek;
	t.k[t.klen].z = pouzite_kameny[i]; /* i s ppadnm JOKERBITEM !!! */

	/* bugfix 03.02.2004 -- chyba byla v tom, e pro olka se zskvala
	 * hodnota zastupujcho psmene, msto nuly! */
	t.k[t.klen++].h = skore_vrat_hodnotu(pouzite_kameny[i]);
	
	t.s[t.slen++] = pokladany;
	
	skorePokladanehoPismene = 
			skore_vrat_hodnotu(pouzite_kameny[i]) * rad_pmult[c]; 
	
        skoreOdehranychPismenCelkem += skorePokladanehoPismene;	
	nasobekPokladanehoSlova *= rad_smult[c];

	if (rad_score[c])
	    skoreKolmehoSlova +=
	       rad_smult[c] * (rad_score[c] + skorePokladanehoPismene);

    }/* end for */
	
    /* Pokud jsou jet vlevo od posledn poloenho kamene dal,
     * zapoti je taky (bugfix).
    */
    while ((odehrany=rad_akt[--c].znak) != EMPTY_TILE) {
	t.s[t.slen++] = odehrany; 	/* psmeno ze slova */
	
	/* Viz bugfix o pr dk ve */
	skoreOdehranychPismenCelkem += rad_akt[c].hodnota;
	
	/*
	skoreOdehranychPismenCelkem += 
	    skore_vrat_hodnotu(znak2index(odehrany));	
	*/
    }

    tah_skoreCelkem = skoreKolmehoSlova + 
	          skoreOdehranychPismenCelkem * nasobekPokladanehoSlova;

    if (kameny == MAX_PISMEN_PRIHRADKA)
	tah_skoreCelkem += EMPTY_RACK_BONUS;
 
    /* Jsme te pln nalevo od nalezenho slova */
    t.p.x   = (o == ZD) ? radek : c+1;
    t.p.y   = (o == ZD) ? c+1 : radek; 	/* o 1 vpravo je zatek slova */
    t.o     = o;
    t.skore = tah_skoreCelkem; 
    revstring(t.s, t.slen);	/* Obra sprvn slovo, postupovali jsme zpt */

 
#ifdef TAHY_DEBUG
    
    for(i=0;i<t.slen;i++)
       testuj[i] = t.s[i];
    testuj[i] = '\0';
    if (!testuj_retezec(testuj))
	fprintf(stderr, "FATAL!!! slovo %s nejni\a\n", testuj);


        printf("------------------------------------\n");
        printf("Skore[%d] = %d\n", POCETTAHU++, t.skore);
        printf("MostLeft = [%d,%d,%s]\n",
                t.p.x, t.p.y, (t.o == ZD) ? "Horizontal" : "Vertical");

        printf("Word = ");
        for(i=0;i<t.slen;i++)
            printf("%c", t.s[i]);
        printf("\nUsed tiles = ");

        for(i=0;i<t.klen;i++)
            printf("([%d,%d,%c,%d])",
                t.k[i].x, t.k[i].y, t.k[i].z, t.k[i].h);
        printf("\n");
#endif
   
    /* Zakzan slova ns nezajmaj */	
    for(i=0;i<t.slen;i++)
       zakazane[i] = t.s[i];
    zakazane[i] = '\0';

    /* A pota nevybr ze zakzanch tah */
    if ( (pouzit_zakazana == TRUE) && 
       ((zakazana_seznam_existuje(zakazane)) == TRUE) ) {
	return;
    }

    /* Pidej jen pi hledn tah, ne pi ohodnocen tahu */
    if (pridej)
	/* Co jsme nalezli ulo do seznamu */
	tahy_seznam_pridej(&t);
    
}/* end eval() */

/* -------------------------------------------------------------------------- */

/* Pro danou pozici zatku slova (p) a orientaci (o) a pro danho hre
 * spoti skre za poloen slovo. Naplnme pomocn struktury tak, abychom
 * mohli pout funkci eval() pro spoten skre. 
*/
GLOBAL int tahy_spocitej_body(int hrac, tpozice p, torientace o) {
    
    /* Bude problm s tm, e pi ukldn kamen na desku se zrove
     * ukldaj do pole 'board[][]', take budeme muset projt 
     * 'rack_placed_tiles_coords[]' a v 'rad_akt[]' vynulovat pslun
     * poloky.
    */

    /* Dle je poteba naplnit pole 'pouzite_kameny[]', tak aby psmena
     * la v obrcenm poad (nutn pro fci 'eval()'.
    */

    tpozice kon = p;
    int sloupec, radek;
    int index;

    /* Budeme plnit */
    kameny = 0;
   
    /* Poteba upravit orientaci pro kolm smr, funkce 
     * tahy_nejlevejsi_pismeno() vrac naopak.
    */
    kon.x = (o == ZD) ? p.x : p.y;
    kon.y = (o == ZD) ? p.y : p.x;
    
    /* Zskej naplnn dky, skre, cross, mult apod. */
    tahy_gen_napln_struktury(o, p.x);
    
    /* Pesu se za toto slovo */
    while(JEPISMENO(kon)) {
    
	/* Je-li to psmeno z racku, uprav(vynuluj) 'rad_akt[]'	*/
	if (rack_is_it_currently_played_tile(kon.x-1, kon.y-1)) {
   
	    /* Nulujeme sprvn sloupec */ 
	    sloupec = (o == ZD) ? kon.y : kon.x;
	    
	    rad_akt[sloupec].znak = EMPTY_TILE;
	    rad_akt[sloupec].hodnota = EMPTY_TILE;

	    /* bugfix - 03.02.2004 - nutno detekovat olka,
	     * aby se nehledalo bodov ohodnocen pro zastupujc
	     * psmeno!
	    */	    
	    index = znak2index(PISMENO(kon));	    

	    /* Mme olka */
	    if (HODNOTA(kon) == 0)
		    index |= JOKERBIT;
		    
	    pouzite_kameny[kameny++] = index; 
	    
	}/* end if currently... */

	kon=ZA(kon, o);

    }/* end while */
    
    /* Nulujeme sprvn sloupec */ 
    sloupec = (o == ZD) ? kon.y : kon.x;
    radek = (o == ZD) ? kon.x : kon.y;
 
    /* Konen zavolej, co jsme potebovali :) */
    eval(sloupec, radek, o, FALSE); 

    return(tah_skoreCelkem); 

}/* end of tahy_spocitej_body() */

/* -------------------------------------------------------------------------- */


/* END_OF_FILE tahy_gen.c */
